name = "deutschebahn"
version = "1.4.0"
description = """Collection of utilities related to German train operator Deutsche Bahn.
Includes bindings for https://github.com/derf/db-fakedisplay, coach order API, list of stations, and more.
Includes unmodified DS 100 station data, licensed under CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/deed), created by Deutsche Bahn AG."""
license = "GPLv3"
author = "Julian Metzler"
author_email = "git@mezgr.de"
requires = ['requests']
url = "https://github.com/Mezgrman/pyDeutscheBahn"
keywords = "train web api json departure arrival times coach order station list db deutsche bahn"